@@ -129,9 +129,9 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
             break
 
     if allow_multiline_lambdas:
-      _SetStronglyConnected(node)
+      _SetExpressionPenalty(node, STRONGLY_CONNECTED)
     else:
-      _SetVeryStronglyConnected(node)
+      _SetExpressionPenalty(node, VERY_STRONGLY_CONNECTED)
 
   def Visit_parameters(self, node):  # pylint: disable=invalid-name
     # parameters ::= '(' [typedargslist] ')'
@@ -139,11 +139,18 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
 
     # Can't break before the opening paren of a parameter list.
     _SetUnbreakable(node.children[0])
-    if not style.Get('DEDENT_CLOSING_BRACKETS'):
+    if not (style.Get('INDENT_CLOSING_BRACKETS') or
+            style.Get('DEDENT_CLOSING_BRACKETS')):
       _SetStronglyConnected(node.children[-1])
 
   def Visit_arglist(self, node):  # pylint: disable=invalid-name
     # arglist ::= argument (',' argument)* [',']
+    if pytree_utils.NodeName(node.children[0]) == 'STAR':
+      # Python 3 treats a star expression as a specific expression type.
+      # Process it in that method.
+      self.Visit_star_expr(node)
+      return
+
     self.DefaultNodeVisit(node)
 
     for index in py3compat.range(1, len(node.children)):
@@ -250,10 +257,7 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
 
           # Bump up the split penalty for the first part of a subscript. We
           # would rather not split there.
-          first_leaf = pytree_utils.FirstLeafNode(node.children[1])
-          penalty = pytree_utils.GetNodeAnnotation(
-              first_leaf, pytree_utils.Annotation.SPLIT_PENALTY, default=0)
-          _SetSplitPenalty(first_leaf, penalty + CONNECTED)
+          _IncreasePenalty(node.children[1], CONNECTED)
         else:
           _SetStronglyConnected(node.children[1], node.children[2])
 
@@ -318,7 +322,8 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
           last_child_node = pytree_utils.LastLeafNode(trailer)
           if last_child_node.value.strip().startswith('#'):
             last_child_node = last_child_node.prev_sibling
-          if not style.Get('DEDENT_CLOSING_BRACKETS'):
+          if not (style.Get('INDENT_CLOSING_BRACKETS') or
+                  style.Get('DEDENT_CLOSING_BRACKETS')):
             last = pytree_utils.LastLeafNode(last_child_node.prev_sibling)
             if last.value != ',':
               if last_child_node.value == ']':
@@ -331,6 +336,16 @@ class _SplitPenaltyAssigner(pytree_visitor.PyTreeVisitor):
           # split the two.
           _SetStronglyConnected(trailer.children[-1])
 
+  def Visit_subscriptlist(self, node):  # pylint: disable=invalid-name
+    # subscriptlist ::= subscript (',' subscript)* [',']
+    self.DefaultNodeVisit(node)
+    _SetSplitPenalty(pytree_utils.FirstLeafNode(node), 0)
+    prev_child = None
+    for child in node.children:
+      if prev_child and pytree_utils.NodeName(prev_child) == 'COMMA':
+        _SetSplitPenalty(pytree_utils.FirstLeafNode(child), 0)
+      prev_child = child
+
   def Visit_subscript(self, node):  # pylint: disable=invalid-name
     # subscript ::= test | [test] ':' [test] [sliceop]
     _SetStronglyConnected(*node.children)
